@@ -42,6 +42,9 @@ import shutil
 import pkgutil
 
 
+# TODO: excel custom format 2 report
+
+
 q2_modules = ("q2rad", "q2gui", "q2db", "q2report")
 const = q2const()
 _ = gettext.gettext
@@ -136,18 +139,24 @@ class Q2RadApp(Q2App):
             self.close()
 
     def open_selected_app(self, go_to_q2market=False):
+        wait = Q2WaitShow(5, "Loading app> ")
+        wait.step("Prepare")
         self.clear_app_info()
+        wait.step("Migrate logic DB")
         self.migrate_db_logic()
+        wait.step("Migrate data DB")
         self.migrate_db_data()
+        wait.step("looking for updates")
         self.update_app_packages()
+        wait.step("Done!")
+        wait.close()
+        self.run_module("manifest")
+        self.run_module("version")
+        if self.app_title:
+            self.set_title(f"{self.app_title}({self.selected_application.get('name', '')})")
+        else:
+            self.set_title(f"{self.selected_application.get('name', '')}")
         self.run_module("autorun")
-        self.set_title(f"{self.app_title}({self.selected_application.get('name', '')})")
-        # DEBUG
-        # self.run_forms()
-        # self.run_queries()
-        # self.run_modules()
-        # self.run_reports()
-        # self.run_app_manager()
 
         if go_to_q2market and (
             max(
@@ -346,10 +355,10 @@ class Q2RadApp(Q2App):
             "update_q2rad",
             ("" if "win" in sys.platform else "#!/bin/bash\n")
             + ("q2rad\\scripts\\activate " if "win" in sys.platform else "source q2rad/bin/activate")
-            + "&& pip install --upgrade q2gui"
-            + "&& pip install --upgrade q2db"
-            + "&& pip install --upgrade q2report"
-            + "&& pip install --upgrade q2rad",
+            + "&& pip install --upgrade --force-reinstall q2gui"
+            + "&& pip install --upgrade --force-reinstall q2db"
+            + "&& pip install --upgrade --force-reinstall q2report"
+            + "&& pip install --upgrade --force-reinstall q2rad",
         )
 
         # create run_q2rad
