@@ -27,6 +27,8 @@ from q2rad import Q2Form
 
 _ = gettext.gettext
 
+# TODO: selected rows removing - bug
+
 
 class Q2RadReport(Q2Report):
     def __init__(self, content=""):
@@ -34,7 +36,7 @@ class Q2RadReport(Q2Report):
         self.load(content)
         self.data["const"] = q2app.q2_app.const
         self.waitbar = None
-        self.data_sources_lenght = {}
+        # self.data_sources_lenght = {}
         self.last_focus_widget = q2app.q2_app.focus_widget()
 
     def prepare_output_file(self, output_file):
@@ -134,6 +136,7 @@ class Q2RadReport(Q2Report):
                         sql = sql.replace(p, f"'{value}'")
                     self.data_cursors[x] = q2cursor(sql)
                     data[x] = self.data_cursors[x].records()
+                    data[x] = [row for row in self.data_cursors[x].records()]
                     # print(q2app.q2_app.db_data.last_sql_error)
 
             return real_worker
@@ -154,8 +157,7 @@ class Q2Reports(Q2Form):
         self.add_control("name", _("Name"), datatype="char", datalen=100, pk="*")
         self.add_control("/")
 
-        self.add_control("anchor", "**", control="label")
-
+        self.add_control("anchor", "**", control="label", nogrid=1)
         self.add_control(
             "content",
             "",
@@ -218,6 +220,7 @@ class Q2Reports(Q2Form):
             self.report_edit_form.form_stack = [w]
             self.anchor.add_widget_below(w)
             w.show()
+            self.report_edit_form.s.comment = self.s.comment
             self.report_edit_form.widget().restore_grid_columns()
 
         if self.crud_mode == "NEW":
@@ -227,6 +230,7 @@ class Q2Reports(Q2Form):
 
     def before_crud_save(self):
         self.s.content = self.report_edit_form.get_content()
+        self.s.comment = self.report_edit_form.s.comment
 
 
 class ReportForm:
@@ -333,7 +337,6 @@ class Q2ReportReport(Q2Form):
                 """
 
         self.current_focus = None
-        # self.current_style = ""
 
         if 1:  # Actions
             actions = Q2Actions()
@@ -353,7 +356,8 @@ class Q2ReportReport(Q2Form):
                     dblclick=self.style_button_pressed,
                 )
 
-                self.add_control("", control="label", tag="pages", actions=actions)
+                self.add_control("", tag="pages", actions=actions)
+                # self.add_control("", control="label", tag="pages", actions=actions)
                 self.add_control("/")
                 if self.add_control("/vr", tag="pages"):
                     self.add_control("anchor", "*", control="label")
@@ -503,6 +507,15 @@ class Q2ReportReport(Q2Form):
                 )
                 self.add_control("/")
 
+            if self.add_control("/f", "Cell"):
+                # self.add_control("", _("Name"))
+                # self.add_control("", _("Format"))
+                self.add_control("name", _("Name"), control="line", disabled=1)
+                self.add_control("format", _("Format"), control="line", disabled=1)
+                self.add_control("data", _("Data"), control="text", disabled=1)
+                self.add_control("/")
+            self.add_control("/s")
+
         self.set_content()
 
     def run_report(self, output_file="html"):
@@ -618,7 +631,7 @@ class Q2ReportReport(Q2Form):
             value_list.append(value_list[2])
         return value_list
 
-    def update_style_bar(self, parent_style, selected_style):
+    def update_style_bar(self, parent_style, selected_style, cell_data=None):
         if selected_style is None:
             return
         if self.current_focus is None:
@@ -657,8 +670,19 @@ class Q2ReportReport(Q2Form):
             w = self.widgets().get(x.replace("-", "_"))
             if w:
                 w.check.set_text(True)
+        self.show_cell_content(cell_data)
         self.lock_status_bar = False
 
+    def show_cell_content(self, cell_data):
+        if cell_data:
+            self.s.data = cell_data.get("data", "")
+            self.s.format = cell_data.get("format", "")
+            self.s.name = cell_data.get("name", "")
+        else:
+            self.s.data = ""
+            self.s.format = ""
+            self.s.name = ""
+
     def set_style_button_text(self, text):
         ReportForm.set_style_button_text(self, text)
 
@@ -1341,7 +1365,8 @@ class Q2ReportRows(Q2Form, ReportForm):
                 tmp[f"{key[0]},{key[1]-1}"] = self.rows_data.cells[cell_key]
                 del self.rows_data.cells[cell_key]
             elif key[1] <= current_column:
-                if self.rows_data.cells[cell_key].get("colspan", 0) > 1:
+                col_span = self.rows_data.cells[cell_key].get("colspan", 0)
+                if col_span > 1 and key[1] + col_span > current_column:
                     tmp[cell_key] = self.rows_data.cells[cell_key]
                     tmp[cell_key]["colspan"] = tmp[cell_key]["colspan"] - 1
 
@@ -1396,7 +1421,8 @@ class Q2ReportRows(Q2Form, ReportForm):
                     tmp[f"{key[0]-1},{key[1]}"] = self.rows_data.cells[cell_key]
                     del self.rows_data.cells[cell_key]
                 elif key[0] <= current_row:
-                    if self.rows_data.cells[cell_key].get("rowspan", 0) > 1:
+                    row_span = self.rows_data.cells[cell_key].get("rowspan", 0)
+                    if row_span > 1 and key[0] + row_span > current_row:
                         tmp[cell_key] = self.rows_data.cells[cell_key]
                         tmp[cell_key]["rowspan"] = tmp[cell_key]["rowspan"] - 1
             self.rows_data.cells.update(tmp)
@@ -1474,24 +1500,30 @@ class Q2ReportRows(Q2Form, ReportForm):
             self.edit_cell_content()
 
     def edit_cell_content(self):
-        form = Q2Form(_("Enter row height"))
-        form.add_control("/h")
+        key = f"{self.rows_sheet.current_row()},{self.rows_sheet.current_column()}"
+        cell_data = self.rows_data.cells.get(key, {})
+        form = Q2Form(_("Edit cell content"))
+        form.add_control("/v")
         form.add_control("content", control="code", data=self.rows_sheet.get_text())
+        form.add_control("format", "Format", control="line", data=cell_data.get("format", ""))
+        form.add_control("name", "Name", control="line", data=cell_data.get("name", ""))
         form.ok_button = 1
         form.cancel_button = 1
 
-        def after_form_show():
-            form.w._ok_button.set_focus()
+        # def after_form_show():
+        # form.w._ok_button.set_focus()
 
-        form.after_form_show = after_form_show
+        # form.after_form_show = after_form_show
         form.run()
         if form.ok_pressed:
-            key = f"{self.rows_sheet.current_row()},{self.rows_sheet.current_column()}"
             if key not in self.rows_data.cells:
                 self.rows_data.cells[key] = {}
-            self.rows_data.cells[key]["data"] = form.s.content
+            cell_data["data"] = form.s.content
+            cell_data["format"] = form.s.format
+            cell_data["name"] = form.s.name
             self.rows_sheet.set_text(form.s.content)
             self._repaint()
+            self.report_report_form.show_cell_content(cell_data)
 
     def edit_row_height(self):
         height = self.rows_sheet.get_cell_text(
@@ -1670,19 +1702,16 @@ class Q2ReportRows(Q2Form, ReportForm):
 
     def can_i_merge(self):
         selection = self.rows_sheet.get_selection()
+        # open up spanned cells
+        for x in selection:
+            if x in self.spanned_cells:
+                for row in range(x[0], x[0]+self.spanned_cells[x][0]):
+                    for col in range(x[1], x[1]+self.spanned_cells[x][1]):
+                        if (row, col) not in selection:
+                            selection.append((row, col))
+
         if len(selection) == 1:
             return False
-        #  megre merged cells
-        # _selection = []
-        # for x in selection:
-        #     if x not in self.spanned_cells:
-        #         continue
-        #     for _row in range(x[0], self.spanned_cells[x][0] + 1):
-        #         for _col in range(x[1], self.spanned_cells[x][1] + 1):
-        #             if _row == x[0] and _col == x[1]:
-        #                 continue
-        #             _selection.append((_row, _col))
-        # selection.extend(_selection)
 
         rows = set([x[0] for x in selection])
         cols = set([x[1] for x in selection])
@@ -1725,7 +1754,10 @@ class Q2ReportRows(Q2Form, ReportForm):
         all_style.update(self.rows_data.cells[cell_key]["style"])
 
         self.report_report_form.focus_changed(self.rows_sheet)
-        self.report_report_form.update_style_bar(all_style, self.rows_data.cells[cell_key]["style"])
+
+        self.report_report_form.update_style_bar(
+            all_style, self.rows_data.cells[cell_key]["style"], self.rows_data.cells[cell_key]
+        )
 
     def rows_sheet_focus_out(self):
         pass
@@ -1781,6 +1813,10 @@ class Q2ReportRows(Q2Form, ReportForm):
                 del self.rows_data.cells[x]["style"]
             if self.rows_data.cells[x].get("data") == "":
                 del self.rows_data.cells[x]["data"]
+            if self.rows_data.cells[x].get("format") == "":
+                del self.rows_data.cells[x]["format"]
+            if self.rows_data.cells[x].get("name") == "":
+                del self.rows_data.cells[x]["name"]
             if self.rows_data.cells[x] == {}:
                 cell2del.append(x)
 
